@@ -245,7 +245,8 @@ instance Pretty CTypeSpec where
     pretty (CIntType _)         = text "int"
     pretty (CLongType _)        = text "long"
     pretty (CFloatType _)       = text "float"
-    pretty (CFloat128Type _)    = text "__float128"
+    pretty (CFloatNType n x _)  = text "_Float" <> text (show n) <>
+                                  (if x then text "x" else empty) 
     pretty (CDoubleType _)      = text "double"
     pretty (CSignedType _)      = text "signed"
     pretty (CUnsigType _)       = text "unsigned"
